# AIAutoPlay Event Manager
# By: Jeckel
# Works Better With AIAutoPlay SDK changes by jdog5000
# 

from CvPythonExtensions import *
import CvUtil
import Popup as PyPopup
import CvScreensInterface
import CvScreenEnums
import CvAlertManager
alert = CvAlertManager.CvAlertManager()

gc = CyGlobalContext()
oGame = CyGame()
oLocalText = CyTranslator()

class CvAIAutoPlayEventManager:
	
	def __init__(self, eventManager):
		self.EventKeyDown=6
		self.eventManager = eventManager

		self.iDefaultAIAutoPlayTurns = gc.getDefineINT("CVAIAUTOPLAY_POPUP_DEFAULT_NUM_AUTO_TURNS")
		self.iAIAutoPlayTurnsLeft = 0
		self.bAIAutoPlayCancelWar = False
                self.bAIAutoPlayCancelVictory = False

		self.iAbdicatePopupID = CvScreenEnums.CVAIAUTOPLAY_ABDICATE_POPUP
		self.iEndAbdicatePopupID = CvScreenEnums.CVAIAUTOPLAY_END_ABDICATE_POPUP
		self.iTurnAbdicatePopupID = CvScreenEnums.CVAIAUTOPLAY_TURN_ABDICATE_POPUP
		self.iMPAnnounceAbdicatePopupID = CvScreenEnums.CVAIAUTOPLAY_MP_ANNOUNCE_ABDICATE_POPUP

		eventManager.addEventHandler("ModNetMessage", self.onModNetMessage)
		eventManager.addEventHandler("kbdEvent", self.onKbdEvent)
		eventManager.addEventHandler("BeginGameTurn", self.onBeginGameTurn)
		eventManager.addEventHandler("changeWar", self.onChangeWar)
		eventManager.addEventHandler("victory", self.onVictory)

		eventManager.setPopupHandler(self.iAbdicatePopupID, ["eventAIAutoPlayAbdicate", self.applyAbdicate, self.beginAbdicate])
		eventManager.setPopupHandler(self.iEndAbdicatePopupID, ["eventAIAutoPlayEndAbdicate", self.applyEndAbdicate, self.beginEndAbdicate])
		eventManager.setPopupHandler(self.iTurnAbdicatePopupID, ["eventAIAutoPlayTurnAbdicate", self.applyTurnAbdicate, self.beginTurnAbdicate])
		#eventManager.setPopupHandler(CvUtil.EventEditCityName, ["EditCityName", self.applyEditCityName, self.beginEditCityName])

	################################################################################################

	def onBeginGameTurn(self, argsList):
		'Called at the beginning of the end of each turn'
		iGameTurn = argsList[0]
		if (oGame.getAIAutoPlay() > 0):
                        self.iAIAutoPlayTurnsLeft -= 1

	def onChangeWar(self, argsList):
		'War Status Changes'
		bIsWar = argsList[0]
		iTeam = argsList[1]
		iRivalTeam = argsList[2]
		if (self.bAIAutoPlayCancelWar) and (bIsWar) and (oGame.getAIAutoPlay() > 0):
                        oGame.setAIAutoPlay(0)

	def onVictory(self, argsList):
		'Victory'
		iTeam, iVictory = argsList
		if (self.bAIAutoPlayCancelVictory) \
                   and (iVictory > -1) and (iVictory < gc.getNumVictoryInfos()) \
                   and (oGame.getAIAutoPlay() > 0):
                        oGame.setAIAutoPlay(0)

	################################################################################################
		
	# This method handles the key input and will bring up the
	# Tile Space Data Popup if the player presses the 'SHIFT + S' key.
	def onKbdEvent(self, argsList):
		'keypress handler - return 1 if the event was consumed'
		eventType,key,mx,my,px,py = argsList
			
		theKey=int(key)

		pPlotCursor = CyMap().plot(px, py)

		pHeadSelectedCity = CyInterface().getHeadSelectedCity()

		if (eventType == self.EventKeyDown):
                        iPlayer = gc.getGame().getActivePlayer()
                        pPlayer = gc.getPlayer(iPlayer)
                        """if (theKey == int(InputTypes.KB_D)) \
                           and (not self.eventManager.bAlt) \
                           and (self.eventManager.bCtrl) \
                           and (self.eventManager.bShift):
                                oGame.toggleDebugMode()
                                alert.debug(iPlayer, "Debug Mode", oGame.isDebugMode())
                                return 1"""
                        if (theKey == int(InputTypes.KB_M)) \
                           and (not self.eventManager.bAlt) \
                           and (self.eventManager.bCtrl) \
                           and (self.eventManager.bShift):
                                #alert.alert(0, "test ok")
                                #CyMessageControl().sendModNetMessage(self.iTurnAbdicatePopupID, -1, -1, -1, -1)
                                #oGame.setAIAutoPlay(1)
                                self.beginTurnAbdicate()
                                return 1
                        elif (theKey == int(InputTypes.KB_X)) \
                           and (not self.eventManager.bAlt) \
                           and (self.eventManager.bCtrl) \
                           and (self.eventManager.bShift):
                                if (oGame.getAIAutoPlay() > 0):
                                        self.beginEndAbdicate()
                                        return 1
                                if (pPlayer.isHuman()) and (pPlayer.isAlive()):
                                        self.beginAbdicate()
                                        return 1
                return 0

	################################################################################################

	def onModNetMessage(self, argsList):
		'Called whenever CyMessageControl().sendModNetMessage() is called - this is all for you modders!'
		iData1, iData2, iData3, iData4, iData5 = argsList
		alert.alert(0, "test ok2")
                if (iData1 == self.iEndAbdicatePopupID):
                        oGame.setAIAutoPlay(0)
                        oPopup = PyPopup.PyPopup(self.iEndAbdicatePopupID, contextType = EventContextTypes.EVENTCONTEXT_ALL)
                        oPopup.setHeaderString(oLocalText.getText("TXT_KEY_AIAUTOPLAY_TURN_OFF", ()))
                        oPopup.setBodyString(oLocalText.getText("TXT_KEY_AIAUTOPLAY_TURN_OFF_TEXT", ()))
                        oPopup.addButton("OK")
                        oPopup.launch(False, PopupStates.POPUPSTATE_IMMEDIATE)
		elif (iData1 == self.iTurnAbdicatePopupID):
                        oGame.setAIAutoPlay(1)
                        if (oGame.isHotSeat()) or (not oGame.isGameMultiPlayer()):
                                pass#return None
                        pPlayer = gc.getPlayer(iData2)
                        oPopup = PyPopup.PyPopup(self.iTurnAbdicatePopupID, contextType = EventContextTypes.EVENTCONTEXT_ALL)
                        oPopup.setHeaderString(oLocalText.getText("TXT_KEY_AIAUTOPLAY_TURN_ABDICATE", ()))
                        oPopup.setBodyString(oLocalText.getText("TXT_KEY_AIAUTOPLAY_TURN_ABDICATE_TEXT",
                                                                (gc.getLeaderHeadInfo(pPlayer.getLeaderType()).getDescription(),
                                                                 pPlayer.getCivilizationAdjective(0),
                                                                 oGame.getGameTurn(),
                                                                 oGame.getGameTurn() + iData3,
                                                                 iData3)))
                        oPopup.addButton("OK")
                        oPopup.launch(False, PopupStates.POPUPSTATE_IMMEDIATE)
		elif (iData1 == self.iMPAnnounceAbdicatePopupID):
                        pPlayer = gc.getPlayer(iData2)
                        """oPopup = PyPopup.PyPopup(self.iMPAnnounceAbdicatePopupID, contextType = EventContextTypes.EVENTCONTEXT_ALL)
                        oPopup.setHeaderString(oLocalText.getText("TXT_KEY_AIAUTOPLAY_MP_ANNOUNCE_ABDICATE", ()))
                        oPopup.setBodyString(oLocalText.getText("TXT_KEY_AIAUTOPLAY_MP_ANNOUNCE_ABDICATE_TEXT",
                                                                (gc.getLeaderHeadInfo(pPlayer.getLeaderType()).getDescription(),
                                                                 pPlayer.getCivilizationAdjective(0),
                                                                 oGame.getGameTurn(),
                                                                 oGame.getGameTurn() + iData3,
                                                                 iData3)))
                        oPopup.addButton("OK")
                        oPopup.launch(False, PopupStates.POPUPSTATE_IMMEDIATE)"""
                        oPopupInfo = CyPopupInfo()
                        oPopupInfo.setButtonPopupType(ButtonPopupTypes.BUTTONPOPUP_PYTHON_SCREEN)
                        oPopupInfo.setText(oLocalText.getText("TXT_KEY_AIAUTOPLAY_MP_ANNOUNCE_ABDICATE_TEXT",
                                                                (gc.getLeaderHeadInfo(pPlayer.getLeaderType()).getDescription(),
                                                                 pPlayer.getCivilizationAdjective(0),
                                                                 oGame.getGameTurn(),
                                                                 oGame.getGameTurn() + iData3,
                                                                 iData3)))
                        oPopupInfo.addPopup(iData2)
		return None

	################################################################################################

        def beginAbdicate(self):
                oPopup = PyPopup.PyPopup(self.iAbdicatePopupID, contextType = EventContextTypes.EVENTCONTEXT_ALL)
                oPopup.setHeaderString(oLocalText.getText("TXT_KEY_AIAUTOPLAY_TURN_ON", ()))
                oPopup.setBodyString(oLocalText.getText("TXT_KEY_AIAUTOPLAY_TURN_ON_TEXT", ()))
                oPopup.addSeparator()
                iGroup = 0
                sText = u""
                if (self.iAIAutoPlayTurnsLeft > gc.getDefineINT("CVAIAUTOPLAY_POPUP_DEFAULT_NUM_AUTO_TURNS")):
                        sText += u"%d" %(self.iAIAutoPlayTurnsLeft)
                else:
                        sText += u"%d" %(gc.getDefineINT("CVAIAUTOPLAY_POPUP_DEFAULT_NUM_AUTO_TURNS"))
                sHelp = u"Number of turns to turn over to AI."
                oPopup.createPythonEditBox(sText, sHelp, iGroup)

                iGroup = 1
                self.AutoTypes = {0: oLocalText.getText("TXT_KEY_AIAUTOPLAY_FULLY", ()),
                                        1: oLocalText.getText("TXT_KEY_AIAUTOPLAY_NO", ())}

                oPopup.createPythonPullDown(oLocalText.getText("TXT_KEY_AIAUTOPLAY_AUTOMATION", ()), iGroup)
                for i in range(0, len(self.AutoTypes)):
                        oPopup.addPullDownString(self.AutoTypes[i], i, iGroup)

                oPopup.popup.setSelectedPulldownID(0, iGroup)

                iGroup = 2
                self.AutoRestricts = {0: "No Restrictions",
                                        1: "Cancel On War Declared",
                                        2: "Cancel On Victory Declared",
                                        3: "All of the Above"}

                """oPopup.createPythonCheckBoxes(len(self.AutoRestricts), iGroup)
                for i in range(0, len(self.AutoRestricts)):
                        tBox = self.AutoRestricts[i]
                        oPopup.setPythonCheckBoxText(i, tBox[0], tBox[1], iGroup)"""
                """oPopup.createPythonRadioButtons(len(self.AutoRestricts), iGroup)
                for i in range(0, len(self.AutoRestricts)):
                        tBox = self.AutoRestricts[i]
                        oPopup.setPythonRadioButtonText(i, tBox[0], tBox[1], iGroup)"""
                oPopup.createPythonPullDown(oLocalText.getText("War Restriction", ()), iGroup)
                for i in range(0, len(self.AutoRestricts)):
                        oPopup.addPullDownString(self.AutoRestricts[i], i, iGroup)

                oPopup.popup.setSelectedPulldownID(0, iGroup)

                oPopup.addSeparator()
                oPopup.addButton("OK")
                oPopup.addButton(oLocalText.getText("TXT_KEY_AIAUTOPLAY_CANCEL", ()))

                oPopup.launch(False, PopupStates.POPUPSTATE_IMMEDIATE)
                """oPopupInfo = CyPopupInfo()
                oPopupInfo.setButtonPopupType(ButtonPopupTypes.BUTTONPOPUP_PYTHON)
                oPopupInfo.setText(CyTranslator().getText("TXT_KEY_POPUP_SELECT_TRAIT", ()))
                oPopupInfo.setData1(iPlayer)
                oPopupInfo.setOnClickedPythonCallback("selectTrait")
                oPopupInfo.addPythonButton(CyTranslator().getText("TXT_KEY_TRAIT_AGGRESSIVE", ()), "")
                oPopupInfo.addPopup(iPlayer)"""
                return None

        def applyAbdicate(self, playerID, userData, popupReturn):
                if (popupReturn.getButtonClicked() == 1):
                        return None

                iPullDown = popupReturn.getSelectedPullDownValue(1)
                if (iPullDown == 1):
                        return None

                iAutoTurns = 0
                if (popupReturn.getEditBoxString(0).isdigit()):
                        iAutoTurns = int(popupReturn.getEditBoxString(0))

                if (iAutoTurns < 1):
                        return None
                oGame.setAIAutoPlay(iAutoTurns)
                self.iAIAutoPlayTurnsLeft = iAutoTurns

                iPullDown = popupReturn.getSelectedPullDownValue(2)
                self.bAIAutoPlayCancelWar = False
                self.bAIAutoPlayCancelVictory = False
                if (iPullDown == 1):
                        self.bAIAutoPlayCancelWar = True
                elif (iPullDown == 2):
                        self.bAIAutoPlayCancelVictory = True
                elif (iPullDown == 3):
                        self.bAIAutoPlayCancelWar = True
                        self.bAIAutoPlayCancelVictory = True
                """if (oGame.isHotSeat()) or (not oGame.isGameMultiPlayer()) or (True):
                        CyMessageControl().sendModNetMessage(self.iMPAnnounceAbdicatePopupID,
                                                             gc.getGame().getActivePlayer(),
                                                             iAutoTurns,
                                                             -1, -1)"""
                return None

	################################################################################################

        def beginEndAbdicate(self):
                CyMessageControl().sendModNetMessage(self.iEndAbdicatePopupID, gc.getGame().getActivePlayer(), -1, -1, -1)
                return None

        def applyEndAbdicate(self, playerID, userData, popupReturn):
                return None

	################################################################################################

        def beginTurnAbdicate(self):
                CyMessageControl().sendModNetMessage(self.iTurnAbdicatePopupID, gc.getGame().getActivePlayer(), 1, -1, -1)
                return None

        def applyTurnAbdicate(self, playerID, userData, popupReturn):
                #oGame.setAIAutoPlay(0)
                return None

	################################################################################################

        def applyEditCityName(self, playerID, userData, popupReturn):	
		'Edit City Name Event'
                #alert.debug(0, "applyEditCityName", True)
		iCityID = userData[0]
		bRename = userData[1]
		player = gc.getPlayer(playerID)
		city = player.getCity(iCityID)
		cityName = popupReturn.getEditBoxString(0)
		if (len(cityName) > 30):
			cityName = cityName[:30]
		city.setName(cityName, not bRename)
		return None

	def beginEditCityName(self, city, bRename):
                #alert.debug(0, "beginEditCityName", True)
		return None
		popup = PyPopup.PyPopup(CvUtil.EventEditCityName, EventContextTypes.EVENTCONTEXT_ALL)
		popup.setUserData((city.getID(), bRename))
		popup.setHeaderString(localText.getText("TXT_KEY_NAME_CITY", ()))
		popup.setBodyString(localText.getText("TXT_KEY_SETTLE_NEW_CITY_NAME", ()))
		popup.createEditBox(city.getName())
		popup.setEditBoxMaxCharCount( 15 )
		popup.launch()
		return None

	################################################################################################
		
		
		
