#
#	FILE:	 Continents.py
#	AUTHOR:  Soren Johnson
#	PURPOSE: Global map script - Civ4's default map script
#-----------------------------------------------------------------------------
#	Copyright (c) 2004, 2005 Firaxis Games, Inc. All rights reserved.
#-----------------------------------------------------------------------------
#

from CvPythonExtensions import *
import CvUtil
import CvMapGeneratorUtil
from CvMapGeneratorUtil import FractalWorld
from CvMapGeneratorUtil import TerrainGenerator
from CvMapGeneratorUtil import FeatureGenerator

def getDescription():
	return "TXT_KEY_MAP_SCRIPT_CONTINENTS_DESCR"
	
def isAdvancedMap():
	"This map should show up in simple mode"
	return 0

# < JEraLimit Mod Start 1/1 >
def getNumCustomMapOptions():
	return 1
	
def getCustomMapOptionName(argsList):
	iOption = argsList[0]
	dOptions = {0: "TXT_KEY_MAP_SCRIPT_JERALIMIT_ERA_LIMIT"}
	sText = unicode(CyTranslator().getText(dOptions[iOption], ()))
	return sText
	
def getNumCustomMapOptionValues(argsList):
        iOption = argsList[0]
	dOptions = {0: CyGlobalContext().getNumEraInfos()}
	return int(dOptions[iOption])
	
def getCustomMapOptionDescAt(argsList):
	[iOption, iSelection] = argsList
	dOptions = {}
	dSelections = {-1: "No Last Era"}
	for i in range(CyGlobalContext().getNumEraInfos()):
                dSelections[i] = str(CyGlobalContext().getEraInfo(i).getTextKey())
        dOptions[0] = dict(dSelections)
	sText = unicode(CyTranslator().getText(dOptions[iOption][iSelection], ()))
	return sText
	
def getCustomMapOptionDefault(argsList):
	iOption = argsList[0]
	try:
                dOptions = {0: CyGlobalContext().getInfoTypeForString(CyGlobalContext().getDefineSTRING("DEFAULT_LAST_ERA"))}
        except:
                dOptions = {0: CyGlobalContext().getNumEraInfos() - 1}
	return dOptions[iOption]

def isRandomCustomMapOption(argsList):
	iOption = argsList[0]
	dOptions = {0: False}
	return dOptions[iOption]

def afterGeneration():
        # MUST setLastEra or the mod won't have any effect
        iLastEra = CyMap().getCustomMapOption(0)
	CyGlobalContext().getGame().setLastEra(iLastEra)
	return None
# < JEraLimit Mod End 1/1 >

def generatePlotTypes():
	NiTextOut("Setting Plot Types (Python Continents) ...")
	fractal_world = FractalWorld()
	fractal_world.initFractal(polar = True)
	return fractal_world.generatePlotTypes(water_percent=75)

def generateTerrainTypes():
	NiTextOut("Generating Terrain (Python Continents) ...")
	terraingen = TerrainGenerator()
	terrainTypes = terraingen.generateTerrain()
	return terrainTypes

def addFeatures():
	NiTextOut("Adding Features (Python Continents) ...")
	featuregen = FeatureGenerator()
	featuregen.addFeatures()
	return 0
